import {
  select,
  csv,
  scaleLinear,
  scaleTime,
  extent,
  axisLeft,
  axisBottom,
  line,
  max,
} from 'd3';

//rendering an html page as an SVG element
const svg = select('svg'); // choosing the HTML element that is SVG

const width = +svg.attr('width'); // Getting the width of the SVG
const height = +svg.attr('height'); // Getting the height of the SVG

// Creating Line graph
const render = (data) => {
  // Title for the chart
  const title =
    'Average Temperature from 2024-01-01 to 2024-01-31';

  // Function to parse dates
  const xValue = (d) => new Date(d.Date);
  const xAxisLabel = 'Date'; // Label for x-axis

  // Function to get temperature values
  const yValue = (d) => d['Average Temperature (°C)'];
  const yAxisLabel = 'Average Temperature (°C)'; // Label for y-axis

  // Margins for the chart
  const margin = {
    top: 60,
    right: 40,
    bottom: 88,
    left: 150,
  };

  // Inner dimensions of the chart
  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top - margin.bottom;

  // Creating a group element within the SVG and translating it to fit margins
  const g = svg
    .append('g')
    .attr(
      'transform',
      `translate(${margin.left},${margin.top})`,
    );

  // Creating x-scale for dates
  const xScale = scaleTime()
    .domain(extent(data, xValue))
    .range([0, innerWidth])
    .nice();

  // Creating x-axis
  const xAxis = axisBottom(xScale)
    .tickSize(-innerHeight)
    .tickPadding(15);

  // Creating y-scale for temperatures
  const yScale = scaleLinear()
    .domain([0, max(data, yValue)])
    .range([innerHeight, 0])
    .nice();

  // Creating y-axis
  const yAxis = axisLeft(yScale)
    .tickSize(-innerWidth)
    .tickPadding(10);

  // Appending y-axis to the chart
  const yAxisG = g.append('g').call(yAxis);
  yAxisG.selectAll('.domain').remove();
  yAxisG
    .append('text')
    .attr('class', 'axis-label')
    .attr('y', -90)
    .attr('x', -innerHeight / 3)
    .attr('fill', 'black')
    .attr('transform', `rotate(-90)`)
    .text(yAxisLabel);

  // Appending x-axis to the chart
  const xAxisG = g
    .append('g')
    .call(xAxis)
    .attr('transform', `translate(0,${innerHeight})`);
  xAxisG.select('.domain').remove();
  xAxisG
    .append('text')
    .attr('class', 'axis-label')
    .attr('y', 70)
    .attr('x', innerWidth / 2)
    .attr('fill', 'black')
    .text(xAxisLabel);

  // Creating line generator function
  const lineGenerator = line()
    .x((d) => xScale(xValue(d)))
    .y((d) => yScale(yValue(d)));

  // Appending line to the chart
  g.append('path')
    .attr('class', 'line_path')
    .attr('d', lineGenerator(data))
    .style('fill', 'none')
    .style('stroke', 'red')
    .style('stroke-width', 4);

  // Adding title to the chart
  g.append('text')
    .attr('class', 'title')
    .attr('y', -10)
    .text(title);
};

// Loading data from CSV file
csv(
  'https://gist.githubusercontent.com/ArchanaGoli/d6dc25de309d6e71ba1b9d86b5383642/raw/0374c61f7d47b96cfb95265fc05ac81c9dd82f0b/gistfile1.txt',
).then((data) => {
  // Parsing data
  data.forEach((d) => {
    d['Average Temperature (°C)'] =
      +d['Average Temperature (°C)']; // Converting temperature to number
    d.Date = new Date(d.Date); // Parsing date string to Date object
  });
  // Calling the render function with the data
  render(data);
